@@ -9,8 +9,8 @@ def list_templates(config):
     print "Available templates:\n"
     for tempname in os.listdir(config.get_template_directory()):
         readme = os.path.join(
-                        config.get_template_directory(), 
-                        tempname, 
+                        config.get_template_directory(),
+                        tempname,
                         'README')
         synopsis = open(readme).next()
         print "%s - %s" % (tempname, synopsis)
@@ -39,44 +39,57 @@ def init(config, directory, template='generic'):
     script = ScriptDirectory(directory)
 
     for file_ in os.listdir(template_dir):
+        file_path = os.path.join(template_dir, file_)
         if file_ == 'alembic.ini.mako':
             config_file = os.path.abspath(config.config_file_name)
             if os.access(config_file, os.F_OK):
                 util.msg("File %s already exists, skipping" % config_file)
             else:
                 script._generate_template(
-                    os.path.join(template_dir, file_),
+                    file_path,
                     config_file,
                     script_location=directory
                 )
-        else:
+        elif os.path.isfile(file_path):
             output_file = os.path.join(directory, file_)
             script._copy_file(
-                os.path.join(template_dir, file_), 
+                file_path,
                 output_file
             )
 
     util.msg("Please edit configuration/connection/logging "\
             "settings in %r before proceeding." % config_file)
 
-def revision(config, message=None, autogenerate=False):
+def revision(config, message=None, autogenerate=False, sql=False):
     """Create a new revision file."""
 
     script = ScriptDirectory.from_config(config)
     template_args = {}
     imports = set()
+
+    environment = util.asbool(
+        config.get_main_option("revision_environment")
+    )
+
     if autogenerate:
+        environment = True
         util.requires_07("autogenerate")
         def retrieve_migrations(rev, context):
             if script.get_revision(rev) is not script.get_revision("head"):
                 raise util.CommandError("Target database is not up to date.")
             autogen._produce_migration_diffs(context, template_args, imports)
             return []
+    elif environment:
+        def retrieve_migrations(rev, context):
+            return []
 
+    if environment:
         with EnvironmentContext(
             config,
             script,
-            fn = retrieve_migrations
+            fn=retrieve_migrations,
+            as_sql=sql,
+            template_args=template_args,
         ):
             script.run_env()
     script.generate_revision(util.rev_id(), message, **template_args)
@@ -111,7 +124,6 @@ def downgrade(config, revision, sql=False, tag=None):
     """Revert to a previous version."""
 
     script = ScriptDirectory.from_config(config)
-
     starting_rev = None
     if ":" in revision:
         if not sql:
@@ -187,7 +199,7 @@ def stamp(config, revision, sql=False, tag=None):
         context._update_current_rev(current, dest)
         return []
     with EnvironmentContext(
-        config, 
+        config,
         script,
         fn = do_stamp,
         as_sql = sql,
@@ -198,9 +210,9 @@ def stamp(config, revision, sql=False, tag=None):
 
 def splice(config, parent, child):
     """'splice' two branches, creating a new revision file.
-    
+
     this command isn't implemented right now.
-    
+
     """
     raise NotImplementedError()
 
