@@ -18,12 +18,12 @@ _impls = {}
 class DefaultImpl(object):
     """Provide the entrypoint for major migration operations,
     including database-specific behavioral variances.
-    
+
     While individual SQL/DDL constructs already provide
     for database-specific implementations, variances here
     allow for entirely different sequences of operations
     to take place for a particular migration, such as
-    SQL Server's special 'IDENTITY INSERT' step for 
+    SQL Server's special 'IDENTITY INSERT' step for
     bulk inserts.
 
     """
@@ -31,8 +31,9 @@ class DefaultImpl(object):
     __dialect__ = 'default'
 
     transactional_ddl = False
+    command_terminator = ";"
 
-    def __init__(self, dialect, connection, as_sql, 
+    def __init__(self, dialect, connection, as_sql,
                     transactional_ddl, output_buffer,
                     context_opts):
         self.dialect = dialect
@@ -55,8 +56,8 @@ class DefaultImpl(object):
     def bind(self):
         return self.connection
 
-    def _exec(self, construct, execution_options=None, 
-                            multiparams=(), 
+    def _exec(self, construct, execution_options=None,
+                            multiparams=(),
                             params=util.immutabledict()):
         if isinstance(construct, basestring):
             construct = text(construct)
@@ -66,7 +67,7 @@ class DefaultImpl(object):
                 raise Exception("Execution arguments not allowed with as_sql")
             self.static_output(unicode(
                     construct.compile(dialect=self.dialect)
-                    ).replace("\t", "    ").strip() + ";")
+                    ).replace("\t", "    ").strip() + self.command_terminator)
         else:
             conn = self.connection
             if execution_options:
@@ -76,7 +77,7 @@ class DefaultImpl(object):
     def execute(self, sql, execution_options=None):
         self._exec(sql, execution_options)
 
-    def alter_column(self, table_name, column_name, 
+    def alter_column(self, table_name, column_name,
                         nullable=None,
                         server_default=False,
                         name=None,
@@ -88,7 +89,7 @@ class DefaultImpl(object):
                     ):
 
         if nullable is not None:
-            self._exec(base.ColumnNullable(table_name, column_name, 
+            self._exec(base.ColumnNullable(table_name, column_name,
                                 nullable, schema=schema,
                                 existing_type=existing_type,
                                 existing_server_default=existing_server_default,
@@ -133,7 +134,7 @@ class DefaultImpl(object):
         self._exec(schema.DropConstraint(const))
 
     def rename_table(self, old_table_name, new_table_name, schema=None):
-        self._exec(base.RenameTable(old_table_name, 
+        self._exec(base.RenameTable(old_table_name,
                     new_table_name, schema=schema))
 
     def create_table(self, table):
@@ -195,8 +196,8 @@ class DefaultImpl(object):
         else:
             return True
 
-    def compare_server_default(self, inspector_column, 
-                            metadata_column, 
+    def compare_server_default(self, inspector_column,
+                            metadata_column,
                             rendered_metadata_default):
         conn_col_default = inspector_column['default']
         return conn_col_default != rendered_metadata_default
@@ -204,30 +205,30 @@ class DefaultImpl(object):
     def start_migrations(self):
         """A hook called when :meth:`.EnvironmentContext.run_migrations`
         is called.
-        
+
         Implementations can set up per-migration-run state here.
-        
+
         """
 
     def emit_begin(self):
         """Emit the string ``BEGIN``, or the backend-specific
         equivalent, on the current connection context.
-        
+
         This is used in offline mode and typically
         via :meth:`.EnvironmentContext.begin_transaction`.
-        
+
         """
-        self.static_output("BEGIN;")
+        self.static_output("BEGIN" + self.command_terminator)
 
     def emit_commit(self):
         """Emit the string ``COMMIT``, or the backend-specific
         equivalent, on the current connection context.
-        
+
         This is used in offline mode and typically
         via :meth:`.EnvironmentContext.begin_transaction`.
-        
+
         """
-        self.static_output("COMMIT;")
+        self.static_output("COMMIT" + self.command_terminator)
 
 class _literal_bindparam(_BindParamClause):
     pass
