@@ -4,11 +4,12 @@ from sqlalchemy.ext.compiler import compiles
 from alembic.ddl.base import alter_table
 from alembic import util
 from sqlalchemy import types as sqltypes
+from sqlalchemy import schema
 
 class MySQLImpl(DefaultImpl):
     __dialect__ = 'mysql'
 
-    def alter_column(self, table_name, column_name, 
+    def alter_column(self, table_name, column_name,
                         nullable=None,
                         server_default=False,
                         name=None,
@@ -23,7 +24,7 @@ class MySQLImpl(DefaultImpl):
                 table_name, column_name,
                 schema=schema,
                 newname=name if name is not None else column_name,
-                nullable =nullable if nullable is not None else 
+                nullable =nullable if nullable is not None else
                                 existing_nullable if existing_nullable is not None
                                 else True,
                 type_=type_ if type_ is not None else existing_type,
@@ -85,4 +86,22 @@ def _mysql_colspec(compiler, name, nullable, server_default, type_):
 
     return spec
 
+@compiles(schema.DropConstraint, "mysql")
+def _mysql_drop_constraint(element, compiler, **kw):
+    """Redefine SQLAlchemy's drop constraint to
+    raise errors for invalid constraint type."""
+
+    constraint = element.element
+    if isinstance(constraint, (schema.ForeignKeyConstraint,
+                                schema.PrimaryKeyConstraint,
+                                schema.UniqueConstraint)
+                                ):
+        return compiler.visit_drop_constraint(element, **kw)
+    elif isinstance(constraint, schema.CheckConstraint):
+        raise NotImplementedError(
+                "MySQL does not support CHECK constraints.")
+    else:
+        raise NotImplementedError(
+                "No generic 'DROP CONSTRAINT' in MySQL - "
+                "please specify constraint type")
 
