@@ -12,6 +12,7 @@ _legacy_rev = re.compile(r'([a-f0-9]+)\.py$')
 _mod_def_re = re.compile(r'(upgrade|downgrade)_([a-z0-9]+)')
 _slug_re = re.compile(r'\w+')
 _default_file_template = "%(rev)s_%(slug)s"
+_relative_destination = re.compile(r'(?:\+|-)\d+')
 
 class ScriptDirectory(object):
     """Provides operations upon an Alembic script directory.
@@ -44,19 +45,21 @@ class ScriptDirectory(object):
 
     @classmethod
     def from_config(cls, config):
-        """Produce a new :class:`.ScriptDirectory` given a :class:`.Config` 
+        """Produce a new :class:`.ScriptDirectory` given a :class:`.Config`
         instance.
 
         The :class:`.Config` need only have the ``script_location`` key
         present.
 
         """
+        script_location = config.get_main_option('script_location')
+        if script_location is None:
+            raise util.CommandError("No 'script_location' key "
+                                    "found in configuration.")
         return ScriptDirectory(
-                    util.coerce_resource_to_filename(
-                        config.get_main_option('script_location')
-                    ),
+                    util.coerce_resource_to_filename(script_location),
                     file_template = config.get_main_option(
-                                        'file_template', 
+                                        'file_template',
                                         _default_file_template)
                     )
 
@@ -90,7 +93,7 @@ class ScriptDirectory(object):
             return self._revision_map[id_]
         except KeyError:
             # do a partial lookup
-            revs = [x for x in self._revision_map 
+            revs = [x for x in self._revision_map
                     if x is not None and x.startswith(id_)]
             if not revs:
                 raise util.CommandError("No such revision '%s'" % id_)
@@ -119,7 +122,7 @@ class ScriptDirectory(object):
     _as_rev_number = as_revision_number
 
     def iterate_revisions(self, upper, lower):
-        """Iterate through script revisions, starting at the given 
+        """Iterate through script revisions, starting at the given
         upper revision identifier and ending at the lower.
 
         The traversal uses strictly the `down_revision`
@@ -130,16 +133,38 @@ class ScriptDirectory(object):
         The iterator yields :class:`.Script` objects.
 
         """
+        if upper is not None and _relative_destination.match(upper):
+            relative = int(upper)
+            revs = list(self._iterate_revisions("head", lower))
+            revs = revs[-relative:]
+            if len(revs) != abs(relative):
+                raise util.CommandError("Relative revision %s didn't "
+                            "produce %d migrations" % (upper, abs(relative)))
+            return iter(revs)
+        elif lower is not None and _relative_destination.match(lower):
+            relative = int(lower)
+            revs = list(self._iterate_revisions(upper, "base"))
+            revs = revs[0:-relative]
+            if len(revs) != abs(relative):
+                raise util.CommandError("Relative revision %s didn't "
+                            "produce %d migrations" % (lower, abs(relative)))
+            return iter(revs)
+        else:
+            return self._iterate_revisions(upper, lower)
+
+    def _iterate_revisions(self, upper, lower):
         lower = self.get_revision(lower)
         upper = self.get_revision(upper)
+        orig = lower.revision if lower else 'base', \
+                upper.revision if upper else 'base'
         script = upper
         while script != lower:
+            if script is None and lower is not None:
+                raise util.CommandError(
+                        "Revision %s is not an ancestor of %s" % orig)
             yield script
             downrev = script.down_revision
             script = self._revision_map[downrev]
-            if script is None and lower is not None:
-                raise util.CommandError(
-                        "Couldn't find revision %s" % downrev)
 
     def _upgrade_revs(self, destination, current_rev):
         revs = self.iterate_revisions(destination, current_rev)
@@ -193,10 +218,19 @@ class ScriptDirectory(object):
         map_[None] = None
         return map_
 
-    def _rev_path(self, rev_id, message):
+    def _rev_path(self, rev_id, message, create_date):
         slug = "_".join(_slug_re.findall(message or "")).lower()[0:20]
         filename = "%s.py" % (
-            self.file_template % {'rev':rev_id, 'slug':slug}
+            self.file_template % {
+                'rev': rev_id,
+                'slug': slug,
+                'year': create_date.year,
+                'month': create_date.month,
+                'day': create_date.day,
+                'hour': create_date.hour,
+                'minute': create_date.month,
+                'second': create_date.second
+            }
         )
         return os.path.join(self.versions, filename)
 
@@ -258,14 +292,14 @@ class ScriptDirectory(object):
     def _generate_template(self, src, dest, **kw):
         util.status("Generating %s" % os.path.abspath(dest),
             util.template_to_file,
-            src, 
+            src,
             dest,
             **kw
         )
 
     def _copy_file(self, src, dest):
-        util.status("Generating %s" % os.path.abspath(dest), 
-                    shutil.copy, 
+        util.status("Generating %s" % os.path.abspath(dest),
+                    shutil.copy,
                     src, dest)
 
     def generate_revision(self, revid, message, refresh=False, **kw):
@@ -288,13 +322,14 @@ class ScriptDirectory(object):
 
         """
         current_head = self.get_current_head()
-        path = self._rev_path(revid, message)
+        create_date = datetime.datetime.now()
+        path = self._rev_path(revid, message, create_date)
         self._generate_template(
             os.path.join(self.dir, "script.py.mako"),
             path,
             up_revision=str(revid),
             down_revision=current_head,
-            create_date=datetime.datetime.now(),
+            create_date=create_date,
             message=message if message is not None else ("empty message"),
             **kw
         )
@@ -370,9 +405,9 @@ class Script(object):
 
     def __str__(self):
         return "%s -> %s%s%s, %s" % (
-                        self.down_revision, 
-                        self.revision, 
-                        " (head)" if self.is_head else "", 
+                        self.down_revision,
+                        self.revision,
+                        " (head)" if self.is_head else "",
                         " (branchpoint)" if self.is_branch_point else "",
                         self.doc)
 
@@ -396,7 +431,7 @@ class Script(object):
                         "Could not determine revision id from filename %s. "
                         "Be sure the 'revision' variable is "
                         "declared inside the script (please see 'Upgrading "
-                        "from Alembic 0.1 to 0.2' in the documentation)." 
+                        "from Alembic 0.1 to 0.2' in the documentation)."
                         % filename)
             else:
                 revision = m.group(1)
