@@ -1,6 +1,6 @@
 from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
     Numeric, CHAR, ForeignKey, DATETIME, \
-    TypeDecorator, CheckConstraint, Unicode,\
+    TypeDecorator, CheckConstraint, Unicode, Enum,\
     UniqueConstraint, Boolean
 from sqlalchemy.types import NULLTYPE
 from sqlalchemy.dialects import mysql
@@ -164,9 +164,10 @@ class ImplicitConstraintNoGenTest(AutogenTest, TestCase):
     def test_boolean_gen_upgrade(self):
         template_args = {}
         autogenerate._produce_migration_diffs(self.context,
-            template_args, set(), _include_only=['sometable'])
+            template_args, set(),
+            include_symbol=lambda name: name == 'sometable')
         eq_(
-            template_args['upgrades'],
+            re.sub(r"u'", "'", template_args['upgrades']),
             "### commands auto generated by Alembic - please adjust! ###\n"
             "    op.create_table('sometable',\n"
             "    sa.Column('id', sa.Integer(), nullable=False),\n"
@@ -181,16 +182,17 @@ class ImplicitConstraintNoGenTest(AutogenTest, TestCase):
 
         template_args = {}
         autogenerate._produce_migration_diffs(self.context,
-            template_args, set(), _include_only=['someothertable'])
+            template_args, set(),
+            )
         eq_(
-            template_args['downgrades'],
+            re.sub(r"u'", "'", template_args['downgrades']),
             "### commands auto generated by Alembic - please adjust! ###\n"
             "    op.create_table('someothertable',\n"
-            "    sa.Column(u'id', mysql.INTEGER(display_width=11), "
+            "    sa.Column('id', mysql.INTEGER(display_width=11), "
                 "nullable=False),\n"
-            "    sa.Column(u'value', mysql.TINYINT(display_width=1), "
+            "    sa.Column('value', mysql.TINYINT(display_width=1), "
             "nullable=True),\n"
-            "    sa.PrimaryKeyConstraint(u'id')\n    )\n"
+            "    sa.PrimaryKeyConstraint('id')\n    )\n"
             "    op.drop_table('sometable')\n"
             "    ### end Alembic commands ###"
         )
@@ -331,6 +333,28 @@ class AutogenerateDiffTest(AutogenTest, TestCase):
     op.drop_table('item')
     ### end Alembic commands ###""")
 
+    def test_include_symbol(self):
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                'compare_type': True,
+                'compare_server_default': True,
+                'target_metadata': self.m2,
+                'include_symbol': lambda name, schema=None:
+                                    name in ('address', 'order'),
+                'upgrade_token': "upgrades",
+                'downgrade_token': "downgrades",
+                'alembic_module_prefix': 'op.',
+                'sqlalchemy_module_prefix': 'sa.',
+            }
+        )
+        template_args = {}
+        autogenerate._produce_migration_diffs(context, template_args, set())
+        assert "alter_column('user'" not in template_args['upgrades']
+        assert "alter_column('user'" not in template_args['downgrades']
+        assert "alter_column('order'" in template_args['upgrades']
+        assert "alter_column('order'" in template_args['downgrades']
+
     def test_skip_null_type_comparison_reflected(self):
         diff = []
         autogenerate._compare_type("sometable", "somecol",
@@ -566,4 +590,20 @@ class AutogenRenderTest(TestCase):
             "existing_server_default='5')"
         )
 
+    def test_render_enum(self):
+        eq_ignore_whitespace(
+            autogenerate._repr_type(
+                        "sa.",
+                        Enum("one", "two", "three", name="myenum"),
+                        self.autogen_context),
+            "sa.Enum('one', 'two', 'three', name='myenum')"
+        )
+        eq_ignore_whitespace(
+            autogenerate._repr_type(
+                        "sa.",
+                        Enum("one", "two", "three"),
+                        self.autogen_context),
+            "sa.Enum('one', 'two', 'three')"
+        )
+
 # TODO: tests for dialect-specific type rendering + imports
